#include <alloc.h>
#include <DOS.H>
#include <MEM.H>

BYTE OldMode;

/*****************************************************************************
	Identifie une carte de type VGA et sauve le mode graphique courant

Valeur retour :
   - TRUE si une carte VGA a ete identifiee.
   - FALSE si la carte n'est pas au moins VGA.
*****************************************************************************/
BOOL InitCard () {

REGS Regs; // Registres du processeur

// Recupere le mode graphique courant ////////////////////////////////////////
   Regs.h.ah = 0x0F; //fonction 0x0F (BIOS VIDEO)
   int86(0x10,&Regs,&Regs);
   OldMode = Regs.h.al;
// Verifie la presence d'une carte VGA ///////////////////////////////////////
   Regs.x.ax = 0x1a00;   /* La fonction 1Ah n'existe que pour VGA */
   int86(0x10, &Regs, &Regs );
   if( Regs.h.al == 0x1a ) return TRUE;
   return FALSE;
}

/*****************************************************************************
                    Initialise le mode graphique demand
parametres :
   - Mode : mode graphique selectionn.
   - Effacement : TRUE pour effacer l'cran.

Commentaires :
   Dans cette routine, l'assembleur n'a pas ete utilise car la vitesse de 
commutation n'est pas un facteur critique, celle ci n'intervenant qu'en 
general une seule fois par programme.
*****************************************************************************/
void InitMode(WORD Mode, BOOL Effacement){

REGS Regs;
// Charge les registres du processeur ////////////////////////////////////////
   Regs.h.ah = 0x00;
   Regs.h.al = Mode;
   if (!Effacement) Regs.h.al |= 0x80;
// Commute le mode graphique /////////////////////////////////////////////////
   int86(0x10,&Regs,&Regs);
}

/*****************************************************************************
                     Rtablit le mode graphique d'origine
*****************************************************************************/
void RestoreMode() {
	InitMode(OldMode,FALSE);
}

/*****************************************************************************
                      Change la couleur du cadre
parametres :
   - Couleur : couleur du cadre.
   - Synchro : si TRUE, le changement de couleur est synchronise avec le 
balayage de l'ecran

Commentaires
   La routine a ete ecrite en assembleur sans pour autant compliquer 
d'avantage, en revanche son execution est tres fortement acceleree.
   La technique de synchronisation des sorties ecrans avec le balayage du 
rayon cathodique est ici montre. Il peut etre utilise dans d'autre 
circonstances. Son fonctionnement sera detaille dans un prochain article.
*****************************************************************************/
void ChangeFrame (BYTE Couleur, BOOL Synchro) {

// Synchronise le changement de couleur avec le balayage ecran ///////////////
   if (Synchro) {
      while((inp(0x03DA) & 8)) {};
      while(!(inp(0x03DA) & 8)) {};
   }
// Change la couleur du cadre ////////////////////////////////////////////////
   asm {
      MOV AH,0x10
      MOV AL,1
      MOV BH, Couleur
      int 0x10
   }
}
